/*******************************************************************************
 * Copyright (c) 2015, 2016 Google, Inc and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *   Stefan Xenos (Google) - Initial implementation
 *******************************************************************************/
package org.aspectj.org.eclipse.jdt.internal.core.nd.java;

import org.aspectj.org.eclipse.jdt.internal.compiler.impl.Constant;
import org.aspectj.org.eclipse.jdt.internal.compiler.impl.LongConstant;
import org.aspectj.org.eclipse.jdt.internal.core.nd.Nd;
import org.aspectj.org.eclipse.jdt.internal.core.nd.field.FieldLong;
import org.aspectj.org.eclipse.jdt.internal.core.nd.field.StructDef;

public final class NdConstantLong extends NdConstant {
	public static final FieldLong VALUE;

	@SuppressWarnings("hiding")
	public static StructDef<NdConstantLong> type;

	static {
		type = StructDef.create(NdConstantLong.class, NdConstant.type);
		VALUE = type.addLong();
		type.done();
	}

	public NdConstantLong(Nd nd, long address) {
		super(nd, address);
	}

	protected NdConstantLong(Nd nd) {
		super(nd);
	}

	public static NdConstantLong create(Nd nd, long value) {
		NdConstantLong result = new NdConstantLong(nd);
		result.setValue(value);
		return result;
	}

	public void setValue(long value) {
		VALUE.put(getNd(), this.address, value);
	}

	public long getValue() {
		return VALUE.get(getNd(), this.address);
	}

	@Override
	public Constant getConstant() {
		return LongConstant.fromValue(getValue());
	}
}
